# coding=utf-8
import re

from django.core.exceptions import ValidationError


def kpp_validator(value):
    errors = []
    if len(value) < 9:
        errors.append(u'Значение должно быть не короче 9 символов')
    if not value.isdigit():
        errors.append(u'Поле должно содержать только цифры')
    if errors:
        raise ValidationError(errors)


def inn_validator(value):
    def inn_csum(value):
        k = (3, 7, 2, 4, 10, 3, 5, 9, 4, 6, 8)
        pairs = zip(k[11-len(value):], [int(x) for x in value])
        return str(sum([k * v for k, v in pairs]) % 11 % 10)

    inn_length = len(value)
    if inn_length not in (10, 12):
        raise ValidationError(u'Должно быть введено 10 или 12 цифр')
    elif inn_length == 10:
        if value[-1] != inn_csum(value[:-1]):
            raise ValidationError(u'Введен некорректный ИНН')
    elif inn_length == 12:
        if value[-2:] != inn_csum(value[:-2]) + inn_csum(value[:-1]):
            raise ValidationError(u'Введен некорректный ИНН')


def ogrn_validator(value):
    errors = []
    if len(value) not in (13, 15):
        errors.append(u'Должно быть введено 13 (ОГРН) или 15 (ОГРНИП) цифр')
    if not value.isdigit():
        errors.append(u'Поле должно содержать только цифры')

    if errors:
        raise ValidationError(errors)

    if value[-1] != str(int(value[:-1]) % (len(value) - 2))[-1]:
        raise ValidationError(u'Введен некорректный ОГРН/ОГРНИП')


def phone_validator(value):
    errors = []
    if re.findall('[a-zA-Zа-яА-я]', value, re.UNICODE):
        errors.append(u'Поле не должно содержать буквы')
    if not re.match('^((\+7|8)[\- ]?)(\(?\d{3,4}\)?[\- ]?)?[\d\- ]{6,10}$', value, re.UNICODE) or not len(re.sub('\D', '', value)) == 11:
        errors.append(u'Номер должен начинаться с +7 или 8 и содержать 11 цифр')
    if errors:
        raise ValidationError(errors)


from django.db.models import fields
from russian_fields import fields as formfields
from russian_fields import validators

__all__ = ('INNField', "KPPField", "OGRNField", "PhoneNumberField")


class KPPField(fields.CharField):
    default_validators = [validators.kpp_validator]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 9)
        super(KPPField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'varchar(%s)' % self.max_length

    def get_internal_type(self):
        return "KPPField"

    def formfield(self, **kwargs):
        defaults = {'form_class': formfields.KPPField}
        defaults.update(kwargs)
        return super(KPPField, self).formfield(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = self.__class__.__module__ + "." + self.__class__.__name__
        args, kwargs = introspector(self)
        # That's our definition!PercentField
        return field_class, args, kwargs


class INNField(fields.CharField):
    default_validators = [validators.inn_validator]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 12)
        super(INNField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'varchar(%s)' % self.max_length

    def get_internal_type(self):
        return "INNField"

    def formfield(self, **kwargs):
        defaults = {'form_class': formfields.INNField}
        defaults.update(kwargs)
        return super(INNField, self).formfield(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = self.__class__.__module__ + "." + self.__class__.__name__
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


class OGRNField(fields.CharField):
    default_validators = [validators.ogrn_validator]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 13)
        super(OGRNField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'varchar(%s)' % self.max_length

    def get_internal_type(self):
        return "OGRNField"

    def formfield(self, **kwargs):
        defaults = {'form_class': formfields.OGRNField}
        defaults.update(kwargs)
        return super(OGRNField, self).formfield(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = self.__class__.__module__ + "." + self.__class__.__name__
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


class PhoneNumberField(fields.CharField):
    default_validators = [validators.phone_validator]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 30)
        super(PhoneNumberField, self).__init__(*args, **kwargs)

    def db_type(self, connection):
        return 'varchar(%s)' % self.max_length

    def get_internal_type(self):
        return "PhoneNumberField"

    def formfield(self, **kwargs):
        defaults = {'form_class': formfields.PhoneNumberField}
        defaults.update(kwargs)
        return super(PhoneNumberField, self).formfield(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = self.__class__.__module__ + "." + self.__class__.__name__
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


from django import forms

from russian_fields import validators

class KPPField(forms.CharField):
    default_validators = [validators.kpp_validator]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(KPPField, self).clean(value)


class INNField(forms.CharField):
    default_validators = [validators.inn_validator]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(INNField, self).clean(value)


class OGRNField(forms.CharField):
    default_validators = [validators.ogrn_validator]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(OGRNField, self).clean(value)


class PhoneNumberField(forms.CharField):
    default_validators = [validators.phone_validator]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(PhoneNumberField, self).clean(value)


class PercentField(forms.IntegerField):
    def to_python(self, value):
        if value and isinstance(value, basestring) and value[-1] == '%':
            value = value[:-1]
        return super(PercentField, self).to_python(value)

    def clean(self, value):
        return super(PercentField, self).clean(value)